/**************************************************************************************************
  Timers.c
**************************************************************************************************/
#include "Timers.h"
#include "Common.h"
#include "USB_Driver.h"



// Variables
int TimerFlag = 0;           // Set each time the millisecond counter reaches 1000
int Msec = 0;                // Millisecond counter

int PanVal = 3600;           // Mid point of servo travel
int TiltVal = 3600;          // Mid-point of servo travel
int StepVal = 16;            // Amount to increment/decrement the PWM period









// The System Timer is used to drive a software pool of individual timers. The System Timer is set
// to interrupt every millisecond. During the interrupt, all software timers are checked and if
// any are found to be active, they are decremented by one.
/*************************************************************************************************/
// Hardware Level Functions
/*************************************************************************************************/
/**************************************************************************************************
  System Timer Interrupt Handler */
void SystemTimerInterrupt() iv IVT_INT_SysTick ics ICS_AUTO {
     if (COUNTFLAG_bit) {              // Interrupts every millisecond
         irqhandler_timer();           // Service timer pool
         
         Msec += 1;                    // Increment millisecond counter
         if (Msec > 999) {             // When counter goes to 1000
             Msec = 0;                 // Set it to 0
             TimerFlag = 1;            // Set 1 second flag
         }
     }
}

/**************************************************************************************************
  Initialize system timer  */
void TimerInit() {
     init_timerpool();                 // Make all timers free
     
     // PLL frequency (system clock) divided by 8000
     //NVIC_SYSTICKRVR = 18750;          // Set reload value register for 150MHz clock
     NVIC_SYSTICKRVR = 15000;          // Set reload value register for 120MHz clock
     
     NVIC_SYSTICKCVR = 0;              // Reset timer counter
     ENABLE_SYSTICKCSR_bit = 1;        // Enable timer
     TICKINT_bit = 1;                  // Enable timer interrupt
}

/**************************************************************************************************
  Timer processing is done if timer flag is set */
void TimerCheck() {
     if (TimerFlag == 1) {
         TimerFlag = 0;                // Reset 1 second flag
         //USB_PutQueue("Tick\r\n");     // Diagnostic
     }
}

//-------------------------------------------------------------------------------------------------




/*************************************************************************************************/
// Software Level Functions
/*************************************************************************************************/
/**************************************************************************************************
File:         Timers.c
Description:  Implements simple timer pool. No timer chaining
Resolution:   +0/-0.001 second

Author: Jari Lahti (jari.lahti@violasystems.com)
Version Info: 22.1.2004 - First version (JaL)
**************************************************************************************************/
//#include "timers.h"


// Variables
struct timers TimerPool[NUMTIMERS];


/**************************************************************************************************
Function:   init_timerpool
Parameters: none
Return val: void
Desc:       Initialize timer pool
**************************************************************************************************/
void init_timerpool(void) {
     unsigned short int i;

     for (i=0; i < NUMTIMERS; i++) {
          TimerPool[i].TimerField = 0;
          TimerPool[i].Free = TRUE;
     }
}

/**************************************************************************************************
Function:   get_timer
Parameters: none
Return val: unsigned short int - returns handle of free timer
Desc:       Gets free timer from the timer pool
**************************************************************************************************/
unsigned short int get_timer(void) {
     unsigned short int i;

     for (i=0; i < NUMTIMERS; i++) {
          if (TimerPool[i].Free == TRUE) {  // If a free timer is found
              TimerPool[i].Free = FALSE;    // Mark it reserved
              return i;                     // Return timer handle
          }
     }

     // Error Check
     USB_PutQueue("TIMERS: No Timers, Resetting..\r\n");
     
     SystemReset();
     return (0);
}

/**************************************************************************************************
Function:   free_timer
Parameters: none
Return val: unsigned short int nbr - number
Desc:       Puts the given timer back to the timer pool
**************************************************************************************************/
void free_timer(unsigned short int nbr) {
     if (nbr > (NUMTIMERS-1))          // Make a simple check
         return;

     TimerPool[nbr].Free = TRUE;
}

/**************************************************************************************************
Function:   set_timer
Parameters: unsigned short int nbr - number
            unsigned long int tout - time out value
Return val: none
Desc:       Initializes given timer to given time-out
**************************************************************************************************/
void set_timer(unsigned short int nbr, unsigned long int tout) {
     if (nbr > (NUMTIMERS-1))          // Make a simple check
         return;

     if (TimerPool[nbr].Free == TRUE)
         return;

     TimerPool[nbr].TimerField = tout;
}

/**************************************************************************************************
Function:   check_timer
Parameters: unsigned short int nbr - number
Return val: unsigned long int - returns the value of given timer
Desc:       Return the value of given timer
**************************************************************************************************/
unsigned long int check_timer(unsigned short int nbr) {
     return TimerPool[nbr].TimerField;
}

/**************************************************************************************************
Function:   irqhandler_timer
Parameters: void
Return val: void
Desc:       Call this function on hardware timer interrupt
**************************************************************************************************/
void irqhandler_timer(void) {
     unsigned short int i;

     for (i=0; i<NUMTIMERS; i++) {
          if (TimerPool[i].Free == FALSE && TimerPool[i].TimerField != 0)
          TimerPool[i].TimerField--;
     }
}
/*************************************************************************************************/







// Basic Timers (TIM6 & TIM7)
/* They may be used as generic timers for time-base generation but they are also specifically
   used to drive the digital-to-analog converter (DAC). In fact, the timers are internally
   connected to the DAC and are able to drive it through their trigger outputs. There are no
   I/O pins associated with these timers. */
/**************************************************************************************************
   Timer6 Interrupt Handler */
void Timer6Interrupt() iv IVT_INT_TIM6_DAC ics ICS_AUTO {
     UIF_TIM6_SR_bit = 0;  // Clear Update Interrupt Flag in TIM6_SR
     //USB_PutQueue("Timer 6 interrupt\r\n");  // Diagnostic
     
     //--------------------------------------------
     //DoSine();  // Generate Sine wave on DAC1 (PA4)
     //--------------------------------------------
}

/**************************************************************************************************
   Initialize Timer6 */
void Timer6_Initialize(void) {
     TIM6EN_bit = 1;           // Enable TIM6 clock in RCC_APB1ENR
     
     /* The counter clock frequency CK_CNT is equal to fCK_PSC / (PSC[15:0] + 1).
        The PSC register contains the value to be loaded into the active prescaler register
        at each update event.
        For a system clock of 120MHz the fCK_PSC is 60,000,000. If (PSC[15:0] is 5999 then
        CK_CNT is 60,000,000 / (5,999 + 1) = 10,000. */

     // To set TIM6 timouts to 1 second
     //TIM6_PSC = 0x176F;        // Set the prescaler register to 5999
     //TIM6_ARR = 0x2710;        // Set the auto-reload register to 10,000
     
     // To set TIM6 timeouts to 100 millisecond
     //TIM6_PSC = 5999;
     //TIM6_ARR = 100;

     // To set TIM6 timouts to 1 millisecond
     //TIM6_PSC = 59;            // Set the prescaler register to 59 (59 + 1 = divide by 60)
     //TIM6_ARR = 1000;          // Set the auto-reload register to 1,000
     
     // To set TIM6 for a 60Hz Sine wave on DAC1 with 1000 points per cycle
     TIM6_PSC = 99;            // Set the prscaler to 99 (99 + 1 = divide by 100)
     TIM6_ARR = 10;            // Set the auto-reload register to 10
     
     TIM6_CR2bits.MMS = 0x2;   // Update event is selected as a trigger output (TRGO) in TIM6_CR2
     CEN_TIM6_CR1_bit = 1;     // Enable counter in TIM6_CR1
     UIE_TIM6_DIER_bit = 1;    // Enable TIM6 interrupts in TIM6_DIER
     NVIC_IntEnable(IVT_INT_TIM6_DAC);  // Allow interrupts in NVIC
}

/**************************************************************************************************
   Timer7 Interrupt Handler */
void Timer7Interrupt() iv IVT_INT_TIM7 ics ICS_AUTO {
     UIF_TIM7_SR_bit = 0;  // Clear Update Interrupt Flag in TIM7_SR
     USB_PutQueue("Timer 7 interrupt\r\n");  // Diagnostic
}

/**************************************************************************************************
   Initialize Timer7 */
void Timer7_Initialize(void) {
     TIM7EN_bit = 1;           // Enable TIM7 clock in RCC_APB1ENR
     TIM7_PSC = 5999;          // Set the prescaler register to divide by 5999+1=6000
     TIM7_ARR = 10000;         // Set the auto-reload register to time out in 1 second
     TIM7_CR2bits.MMS = 0x2;   // Update event is selected as a trigger output (TRGO) in TIM7_CR2
     CEN_TIM7_CR1_bit = 1;     // Enable counter in TIM7_CR1
     UIE_TIM7_DIER_bit = 1;    // Enable TIM7 interrupts in TIM7_DIER
     NVIC_IntEnable(IVT_INT_TIM7);  // Allow interrupts in NVIC
}





/*************************************************************************************************/
// Advanced-control Timers (TIM1 & TIM8) - Not implemented











/**************************************************************************************************
   General-purpose Timers (TIM2 to TIM5)

   Timer 3 Channel 1 on PB4 is used for the Pan motor PWM signal
   Timer 3 Channel 2 on PB5 is used for the Tilt motor PWM signal

**************************************************************************************************/

void Timer3_Initialize(void){

     unsigned int Ratio;

     Ratio = PWM_TIM3_Init(50);  // Set frequency to 50Hz or period to 20ms
     
     PWM_TIM3_Set_Duty(ratio/2, _PWM_NON_INVERTED, _PWM_CHANNEL1);  // Set duty cycle to one-half of period
     PWM_TIM3_Start(_PWM_CHANNEL1, &_GPIO_MODULE_TIM3_CH1_PB4);
     
     PWM_TIM3_Set_Duty(ratio/2, _PWM_NON_INVERTED, _PWM_CHANNEL2);
     PWM_TIM3_Start(_PWM_CHANNEL2, &_GPIO_MODULE_TIM3_CH2_PB5);
     
     // The PWM frequency is 60000000/25/480000=50Hz or 20ms
     TIM3_PSC = 24;              // Set the prescale register to divide by 24+1=25
     TIM3_ARR = 48000;           // Set the auto-load register to divide by 48000
     
     // The desired PWM pulse period in ms is PP/20*48000
     // The center point for the servo is 1.5ms
     TIM3_CCR1 = PanVal;         // The count to be loaded into TIM3_CCR1 is 1.5/20*48000=3600
     TIM3_CCR2 = TiltVal;        // Same for TIM3_CCR2
     CEN_TIM3_CR1_bit = 1;       // Enable counter
}

/* The servo can only go -/+ 90 degrees. At the extreme travel positions, according to the
   specifications, the PWM period will be 2160 counts for 0.9ms or 5040 counts for 2.1ms.
   Experimentation showed that the sero could go a bit further as indicated in the following code.
*/

int IncPan(unsigned int Step){

    if (PanVal < 6120){
        PanVal += Step;
        TIM3_CCR1 = PanVal;
        return 0;
    }else{
        USB_PutQueue("Pan servo at max\r\n");
        return -1;
    }
}

int DecPan(unsigned int Step){

    if (PanVal > 1080){
        PanVal -= Step;
        TIM3_CCR1 = PanVal;
        return 0;
    }else{
        USB_PutQueue("Pan servo at min\r\n");
        return -1;
    }
}

int HomePan(void){
    PanVal = 3600;                // Center position
    TIM3_CCR1 = PanVal;
    return 0;
}

int SetPan(int Value){            // Value can be positive or negative

    if (Value > 6120){
        PanVal = 6120;            // Limit a positive Value
    } else if (Value < 1080){
        PanVal = 1080;            // Limit a negative Value
    } else {
        PanVal = Value;           // Can safely use Value
    }

    TIM3_CCR1 = PanVal;           // Set pulse width
    return 0;
}



int IncTilt(unsigned int Step){

    if (TiltVal < 6200){
        TiltVal += Step;
        TIM3_CCR2 = TiltVal;
        return 0;
    }else{
        USB_PutQueue("Tilt servo at max\r\n");
        return -1;
    }
}

int DecTilt(unsigned int Step){

    if (TiltVal > 1080){
        TiltVal -= Step;
        TIM3_CCR2 = TiltVal;
        return 0;
    }else{
        USB_PutQueue("Tilt servo at min\r\n");
        return -1;
    }
}

int HomeTilt(void){
    TiltVal = 3600;               // Center position
    TIM3_CCR2 = TiltVal;
    return 0;
}

int SetTilt(int Value){           // Value can be positive or negative

    if (Value > 6120){
        TiltVal = 6120;           // Limit a positive Value
    } else if (Value < 1080){
        TiltVal = 1080;           // Limit a negative Value
    } else {
        TiltVal = Value;          // Can safely use Value
    }

    TIM3_CCR2 = TiltVal;          // Set pulse width
    return 0;
}












/**************************************************************************************************
* End of File
**************************************************************************************************/