/**************************************************************************************************
  USB_Driver
  This module is used for communications with the user through a custom PC program.
**************************************************************************************************/
#include "USB_Driver.h"
#include "Buffers.h"
#include "Timers.h"



// Variables
char readbuff[64];
char writebuff[64];

USB_TxStruct USB_Queue;        // Transmit buffer FIFO and input/output pointers

extern unsigned int PanVal;
extern unsigned int TiltVal;

/**************************************************************************************************
  This is the USB interrupt handler */

void USBInterrupt() iv IVT_INT_OTG_FS ics ICS_AUTO {
     USB_Interrupt_Proc();
}

/**************************************************************************************************
  Initialization */
void USB_Init(void) {
     //USBIE_bit = 0;
     //IPC11bits.USBIP = 7;
   
     HID_Enable(&readbuff,&writebuff);
     USB_TxQueueInit();
     //NVIC_IntEnable(IVT_INT_OTG_FS);  // Enable USB device interrupt
}

/**************************************************************************************************
  This routine initializes the transmit buffer FIFO in and out pointers */
void USB_TxQueueInit(void) {
     USB_Queue.in = 0;
     USB_Queue.out = 0;
}

/**************************************************************************************************
  This routine makes an entry into the transmit FIFO and updates the in pointer to the next
  available location. */
void USB_PutQueue(char *msg) {
     USB_Queue.q[USB_Queue.in++] = msg;
     if (USB_Queue.in > (USB_Queue_Size-1))
         USB_Queue.in = 0;
}

/**************************************************************************************************
  This routine checks the transmit buffer FIFO and, if it is not empty, sends the buffer to USB.
  If the buffer was successfully sent, the out pointer is updated to the next available location.
  */

// Call this function in main loop
void USB_CheckOut(void) {
     //char *USB_XmtBuf;
     
     if (USB_Queue.in != USB_Queue.out) {          // Look for a pointer in the FIFO

          if (HID_Write(USB_Queue.q[USB_Queue.out],64) != 0) {
              USB_Queue.out++;
              if (USB_Queue.out > (USB_Queue_Size-1))  // If past the end of array
                  USB_Queue.out = 0;                   // set pointer to start of array
          }
     }
}

/**************************************************************************************************
  This routine checks USB for a receive message then decodes the first character for a command. */

// Call this function in main loop
void USB_CheckIn(void) {
     char *BufNum;
     char BuffA[8];
     char BuffB[8];
     int NumberA;
     int NumberB;
     int i,j;
     char In;

     if (HID_Read() != 0) {
          switch (readbuff[0]) {
               
               case 'a':  // Test to make sure USB is working
                   BufNum = GetBuffer();
                   sprintf(BufNum,"Command a received\r\n");
                   USB_PutQueue(BufNum);
                   break;
               case 'b': // Display registers
                   BufNum = GetBuffer();
                   sprintf(BufNum,"TIM3_PSC: %04X  TIM3_ARR: %04X\r\n",TIM3_PSC,TIM3_ARR);
                   USB_PutQueue(BufNum);
                   break;
               case 'c':  // Increment Pan servo
                   IncPan(28);
                   BufNum = GetBuffer();
                   sprintf(BufNum,"PanVal: %d\r\n",PanVal);
                   USB_PutQueue(BufNum);
                   break;
               case 'd':  // Decrement Pan servo
                   DecPan(28);
                   BufNum = GetBuffer();
                   sprintf(BufNum,"PanVal: %d\r\n",PanVal);
                   USB_PutQueue(BufNum);
                   break;
               case 'e':  // Home Pan servo
                   HomePan();
                   BufNum = GetBuffer();
                   sprintf(BufNum,"PanVal: %d\r\n",PanVal);
                   USB_PutQueue(BufNum);
                   break;
               case 'f':  // Increment Tilt servo
                   IncTilt(28);
                   BufNum = GetBuffer();
                   sprintf(BufNum,"TiltVal: %d\r\n",TiltVal);
                   USB_PutQueue(BufNum);
                   break;
               case 'g':
                   DecTilt(28);  // Decrement Tilt servo
                   BufNum = GetBuffer();
                   sprintf(BufNum,"TiltVal: %d\r\n",TiltVal);
                   USB_PutQueue(BufNum);
                   break;
               case 'h':  // Home Tilt servo
                   HomeTilt();
                   BufNum = GetBuffer();
                   sprintf(BufNum,"TiltVal: %d\r\n",TiltVal);
                   USB_PutQueue(BufNum);
                   break;
               case 'i':
                   break;
               case 'j':
                   break;
               case 'k':
                   break;
               case 'l':
                   break;
               case 'm':
                   i = 0;
                   j = 1;
                   memset(BuffA,0,8);
                   do {
                       In = BuffA[i++] = readbuff[j++];
                   } while (In != ' ');
                   NumberA = atoi(BuffA);

                   i = 0;  // j is allowed to continue incrementing
                   memset(BuffB,0,8);
                   do {
                       In = BuffB[i++] = readbuff[j++];
                   } while (In != ' ');
                   NumberB = atoi(BuffB);

                   // NumberA and NumberB can be maximum 90, minimum -90
                   NumberA = NumberA * 28 + 3600;
                   SetPan(NumberA);
                   
                   NumberB = NumberB * 28 + 3600;
                   SetTilt(NumberB);

                   BufNum = GetBuffer();
                   sprintf(BufNum,"%d %d\r\n",NumberA,NumberB);
                   USB_PutQueue(BufNum);
                   break;
               case 'n':
                   break;
               case 'o':
                   break;
               case 'p':
                   break;
               case 'q':
                   break;
               case 'r':
                   break;
               case 's':
                   break;
               case 't':
                   break;
               case 'u':
                   break;
               case 'v':
                   break;
               case 'w':
                   break;
               case 'x':
                   break;
               case 'y':
                   break;
               case 'z':
                   break;
               
               // Etc.
              
               default:
                   USB_PutQueue("Unknown command\r\n");
                   
                   BufNum = GetBuffer();  // Get a free buffer
                   (void)sprintf(BufNum,"%c\r\n",readbuff[0]);
                   USB_PutQueue(BufNum);
                   break;
          }
     }
}